#!/bin/bash

#############################################
#             Private functions             #
#############################################

function iana_reserved()
{
	is_nat=$(echo $1 | sed -n \
		-e 's/^10\.\(.*\)$/\1/p' \
		-e 's/^192\.168\.\(.*\)$/\1/p' \
		-e 's/^172\.1[6-9]\.\(.*\)$/\1/p' \
		-e 's/^172\.2[0-9]\.\(.*\)$/\1/p' \
		-e 's/^172\.3[0-1]\.\(.*\)$/\1/p' 2>/dev/null)

	return $(test "x$is_nat" != "x"; echo $?)
}

function __update_kamailio_localnets()
{
	# array of networks
	unset NETS
	unset PATTERNS
	unset HASH
	declare -a NETS
	declare -a PATTERNS
	declare -a PATTERNS_CLEAN

	local addr
	local tmp
	local i

	local wmsdb=/var/www/wmsdb
	local kamcfg_dir=/rw2/etc/kamailio

	local kamcfg_host=${kamcfg_dir}/host_specific.cfg
	local kampid=/var/run/kamailio/sipproxy.pid
	local kamrestart=0

	local ftmp=$(mktemp)

	[ -f "$ftmp" ] && rm --preserve-root $ftmp

	# Add loobpack and rfc1918 networks manually
	NETS+=("127.0.0.0/24")
	NETS+=("10.0.0.0/8")
	NETS+=("172.16.0.0/12")
	NETS+=("192.168.0.0/16")

	# Collect directly connected networks (our l2 segments)
	for addr in $(ip addr show up scope global | awk '/inet[^6]/&&$0!~/host lo| tun| wtun/{print $2}'); do
		NETS+=($addr)
	done

	# Collenct networks from acl-directrtp.conf
	if [ -s /rw2/etc/callweaver/acl-directrtp.conf ]; then
		for addr in $(cat /rw2/etc/callweaver/acl-directrtp.conf | awk -F "=" '{print $2}'); do
			NETS+=($addr)
		done
	fi;

	# Process Local Networks configured in WMS
	for addr in $(sqlite3 $wmsdb "select value from system where variable='NOREWRITE_NETWORKS'" | tr ',' "\n"); do
		NETS+=($addr)
	done

	# Convert NETS array into PATTERNS array
	# WMS-4124. Here and below skip pattern logic. Do not remove obsolete code in tag (to have as less as possible changes for stable)
	# refactored in the trunk.
	for ((i = 0; i < ${#NETS[@]}; ++i)); do
		PATTERNS+=(`ipcalc -n -p ${NETS[$i]} |sed "s/^NETWORK=//g;s/^PREFIX=//g;s/\n//" |paste -d/ -s`)
	done

	# Sort and remove duplicates:
	# - print all array elemets delimited by space
	# - substitute space to a new line
	# - sort and remove duplicates
	# - finally substitute new line chars back into spaces
	# - form a new array with sorted and unique elements
	PATTERNS_CLEAN=($(printf '%s\n' "${PATTERNS[@]}" | tr [:space:] '\n' | sort -u | tr '\n' ' '))

	for ((i = 0; i < ${#PATTERNS_CLEAN[@]}; ++i)); do
		patterns+="${PATTERNS_CLEAN[$i]}"

		if [ $i -lt $((${#PATTERNS_CLEAN[@]} - 1)) ]; then
			patterns+=","
		fi
	done

	ftmp=$(mktemp)
	sed -e '/modparam("via_tool", "localnet", "\(.*\)"/ c modparam("via_tool", "localnet", "'$patterns'")' $kamcfg_host > $ftmp

	if [ -n "$(diff $ftmp $kamcfg_host -q)" ]; then
		if [ "$(cat $ftmp | wc -l )" -gt 0 ]; then
			cat $ftmp > $kamcfg_host
			kamrestart=1
		fi
	fi

	[ -f "$ftmp" ] && rm --preserve-root $ftmp

	if [ "x$gALLOW_RESTART" == "xno" ]; then
		kamrestart=0
	elif [ "x$gALLOW_RESTART" == "xforce" ]; then
		kamrestart=1
	fi

	if [ $kamrestart -eq 1 -a -f "$kampid" ]; then
		echo "Sipproxy configuration updated. Restarting sipproxy."
		/usr/sbin/restart_kamailio_with_delay > /dev/null 2>&1
	else
		echo "Sipproxy config updated without restart."
	fi
}

#############################################
#               Entry point                 #
#############################################

# @arg1 command [required]
#       domain    - update *.wildixin.com domain
#       localnets - update local networks pattern to avoid external IP usage for them
# @arg2 allow_restart [optional] default "yes"
#       yes   - kamailio restart is allowed if required
#       no    - kamailio restart is forbidden even if required
#       force - kamailio restart is forced even if config won't be changed

gCMD=$1
gALLOW_RESTART=$2

case $gCMD in
	"localnets")
		__update_kamailio_localnets
		;;
esac
